"""Constants shared across microservices."""


# Common configuration constants
class Config:
    """Configuration constants used across services."""

    # App config module path
    APP_CONFIG_MODULE = "app.config"
